#ifndef LIGHT_H
#define LIGHT_H

#include "object.h"
#include "color.h"
#include "hit.h"

class Scene;

class Light : public Object
{
public :
  Color color() const { return color_; }
  void setColor(const Color& color) { color_ = color; }

  virtual void initFromDOMElement(const QDomElement& e);
  virtual Color illuminatedColor(const qglviewer::Vec& eye, const Hit& hit) const=0;
  virtual void draw() const =0;

  // Rayhit parameter for translucent objects
  virtual bool visibleFrom(const qglviewer::Vec& pos, const Scene* const scene, Hit &rayhit) const=0;

  virtual float boundingRadius() const { return 0.0f;}

  // Methode principale d'intersection entre l'objet et un rayon
  virtual bool intersect(const Ray&, Hit& hit)const {return false;}

private:
  Color color_;
};


class AmbientLight : public Light
{
public :
	virtual Color illuminatedColor(const qglviewer::Vec& eye, const Hit& hit) const ;
	virtual void draw() const;
	virtual bool visibleFrom(const qglviewer::Vec& pos, const Scene* const scene, Hit &rayhit) const ;
};


class DirectionalLight : public Light
{
public :
  DirectionalLight();

  qglviewer::Vec direction() const { return direction_; }
  void setDirection(const qglviewer::Vec& direction) { direction_ = direction; direction_.normalize(); }

  virtual void initFromDOMElement(const QDomElement& e);
  virtual Color illuminatedColor(const qglviewer::Vec& eye, const Hit& hit) const;
  virtual void draw() const;
  virtual bool visibleFrom(const qglviewer::Vec& pos, const Scene* const scene, Hit &rayhit) const;

private:
  qglviewer::Vec direction_;
};


class PointLight : public Light
{
public :
  qglviewer::Vec position() const { return position_; }
  void setPosition(const qglviewer::Vec& position) { position_ = position;}

  virtual void initFromDOMElement(const QDomElement& e);
  virtual Color illuminatedColor(const qglviewer::Vec& eye, const Hit& hit) const;
  virtual bool visibleFrom(const qglviewer::Vec& pos, const Scene* const scene, Hit &rayhit) const;
  virtual void draw() const;

private:
  qglviewer::Vec position_;
};

#endif // LIGHT_H
